//  Demo 14: MovingSquareThread.java
//
//     Another Simple Example of a Java Thread for Multi-Tasking.
//
//             by Charles W. Neville
//      Copyright Charles W. Neville, July, 1999
//
// A MovingSquareThread object animates a Square object in a 
// separate Thread of execution.  In the animation sequence, the
// Square moves across the screen.

// Like FirstMovingSquareThread.java, but without the animation
// flicker problem!

// The flicker problem we encountered with multiple 
// FirstMovingSquareThread objects in ThreadPracticeB.java occurred 
// because each FirstMovingSquareThread did its background drawing in
// a SEPARATE Graphics buffer.  Thus each object repeatedly overwrote 
// what the previous object had just finished drawing, background and 
// all.  The solution to this problem is simple: just pass a common 
// Graphics buffer object to each MovingSquareThread object via a
// common Image buffer object.

import java.awt.*;
import Square;

// A MovingSquareThread object should be
//
//   1. A Thread so it can run as a separate Thread of execution.
//
// A MovingSquareThread object should know
//
//   1. The Square object it is going to animate (state variable).
//   2. Its parent object, eg. the Frame or Applet Panel on which 
//        the animation will be displayed (state variable).
//   3. The Graphics and Image buffer objects it is going to use
//        for background drawing (state variables). 
//   4. The vector direction in which the Square is going to move 
//        (state variables).
//   5. The distance the Square is going to move (state variable).
//   6. How to set and return the Square object (methods).
//   7. How to set and return the parent object (methods).
//   8. How to set and return the Image buffer object (methods).
//   9. How to set and return the vector direction (methods).
//  10. How to set and return the distance (methods).
//  11. How to run itself (method -- all Threads have to have a
//        run method).  Here is where the animation will actually
//        be done.
//
// A MovingSquareThread object
//
//   1. Is NOT a stand alone application, so it does not need a
//      main().
//   2. Is designed to be used by other classes.

public class MovingSquareThread extends Thread {

   private Square s;    // the Square to be animated, to be moved
                        // diagonally across the screen.

   private Component parent;  // the Canvas, Frame or Applet Panel
                              // on which we draw the animation

   private Graphics g;  // our parent's Graphics object, on which 
                        // we draw

   private int rx, ry;  // the x and y components of the vector 
                        // (direction) the Square is to move
 
   private int d;       // the distance, in number of vector lengths,
                        // the Square is to move

   private Graphics gBuffer;   // for double buffering, as usual,
   private Image imageBuffer;  // but this time we will pass them
                               // in via the Constructor

   // Constructor: as usual, just initializes variables.  The
   // parameters are:
     //  Square s   : the Square to be animated
     //  Component parent : the Canvas, Frame or Applet Panel
     //                     on which we draw the animation
     //  Image imageBuffer : the Image buffer object we use for
     //                      background drawing
     //  int rx, ry : the x and y components of the vector
     //               (direction) in which the Square will move
     //  int d      : the number of steps to move
   public MovingSquareThread(Square s, Component parent, 
                             Image imageBuffer, int rx, int ry,
                             int d) {

       this.s = s;
       this.parent = parent;

       // we can initialize the imageBuffer and gBuffer variables
       // here in the constructor, rather than doing it in init(),
       // because THEY already exist, even if we don't.
       this.imageBuffer = imageBuffer;
       this.gBuffer = imageBuffer.getGraphics();

       this.rx = rx;
       this.ry = ry;
       this.d = d;

   } // Constructor MovingSquareThread

   // init(): initializes the Graphics object we draw on.  
   //         Remember, we can't do this in our Constructor, 
   //         because we have to exist before a call to 
   //         parent.getGraphics() will work.
   //
   //         This is an example of an "internal method".  It will
   //         be called from run().  Therefore, it is declared to 
   //         be private.
   private void init() {

     // initialize g to the Graphics object (context) of our parent.
     // g is now the g of our parent's paint(Graphics g) method.
     g = parent.getGraphics();

     // no need to call repaint() because we are not going to
     // do our drawing in paint()!

   } // init()

   // set our Square object
   public void setSquare(Square s) {

      this.s = s;

   } // setSquare()

   // return our Square object
   public Square getSquare() {

      return s;

   } // getSquare()

   // set our parent object
   public void setParent(Component parent) {

      this.parent = parent;

   } // setParent()

   // return our Parent object
   public Component getParent() {

      return parent;

   } // getParent()

   // set our Image buffer object
   public void setImageBuffer(Image imageBuffer) {

      this.imageBuffer = imageBuffer;

   } // setImageBuffer()

   // return our Image buffer object
   public Image getImageBuffer(Image imageBuffer) {

      return imageBuffer;

   } // getImageBuffer()

   // set our vector x component
   public void setRx(int rx) {

      this.rx = rx;

   } // setRx()

   // return our vector x component
   public int getRx() {

      return rx;

   } // getRx()

   // set our vector y component
   public void setRy(int ry) {

      this.ry = ry;

   } // setRy()

   // return our vector y component
   public int getRy() {

      return ry;

   } // getRy()

   // set our distance
   public void setDistance(int d) {

      this.d = d;

   } // setDistance()

   // return our distance
   public int getDistance() {

      return d;

   } // getDistance()

   // Here is where we do our assigned task, animate the Square!
   // All Threads have to have a run method.
   public void run() {

      Color bg = parent.getBackground();  // background color

      init();  // initialize Graphics and Image buffers

      // This is interesting.  For the first time, we are going to
      // do our drawing outside of the paint() method.  This is so
      // we can continually draw in one Thread of execution while
      // our creator application (a Frame or an Applet) does other
      // things like event processing.  Also, we don't have a 
      // paint() method because we are not a Frame or Applet Panel.

      // Here comes the animation!
      for(int i = 1; i <= d; i++) {

         // move -- do drawing offscreen on the Graphics
         //         buffer object
         s.draw(gBuffer, bg);  // hide square
         s.move(rx, ry);
         s.draw(gBuffer);

         //draw completed image on the screen
         g.drawImage(imageBuffer, 0, 0, parent);

         // To pause, so the animation doesn't go too fast, use
         // the sleep() method of the Thread class.  Unfortunately,
         // it throws an InterruptedException, so we have to embed
         // our simple pause statement in a try -- catch clause.
         try {
           Thread.sleep(20);  // 20 ms pause
         }
         catch (InterruptedException e) {
           // do nothing
         }
       }

   } // run()

} // class MovingSquareThread

   